package javax.activation;

import java.net.URL;
import java.io.InputStream;
import java.io.IOException;
import java.io.File;
import com.sun.activation.registries.LogSupport;
import java.util.Vector;
import com.sun.activation.registries.MimeTypeFile;

public class MimetypesFileTypeMap extends FileTypeMap
{
    private MimeTypeFile[] DB;
    private static final int PROG = 0;
    private static String defaultType;
    
    public MimetypesFileTypeMap() {
        Vector dbv = new Vector(5);
        MimeTypeFile mf = null;
        dbv.addElement(null);
        LogSupport.log("MimetypesFileTypeMap: load HOME");
        try {
            String user_home = System.getProperty("user.home");
            if (user_home != null) {
                String path = user_home + File.separator + ".mime.types";
                mf = this.loadFile(path);
                if (mf != null) {
                    dbv.addElement(mf);
                }
            }
        }
        catch (SecurityException ex) {}
        LogSupport.log("MimetypesFileTypeMap: load SYS");
        try {
            String system_mimetypes = System.getProperty("java.home") + File.separator + "lib" + File.separator + "mime.types";
            mf = this.loadFile(system_mimetypes);
            if (mf != null) {
                dbv.addElement(mf);
            }
        }
        catch (SecurityException ex2) {}
        LogSupport.log("MimetypesFileTypeMap: load JAR");
        this.loadAllResources(dbv, "META-INF/mime.types");
        LogSupport.log("MimetypesFileTypeMap: load DEF");
        mf = this.loadResource("/META-INF/mimetypes.default");
        if (mf != null) {
            dbv.addElement(mf);
        }
        dbv.copyInto(this.DB = new MimeTypeFile[dbv.size()]);
    }
    
    private MimeTypeFile loadResource(String name) {
        InputStream clis = null;
        try {
            clis = SecuritySupport.getResourceAsStream(this.getClass(), name);
            if (clis != null) {
                MimeTypeFile mf = new MimeTypeFile(clis);
                if (LogSupport.isLoggable()) {
                    LogSupport.log("MimetypesFileTypeMap: successfully loaded mime types file: " + name);
                }
                return mf;
            }
            else {
                if (LogSupport.isLoggable()) {
                    LogSupport.log("MimetypesFileTypeMap: not loading mime types file: " + name);
                }
                return null;
            }
        }
        catch (IOException e) {
            if (LogSupport.isLoggable()) {
                LogSupport.log("MimetypesFileTypeMap: can't load " + name, e);
            }
        }
        catch (SecurityException sex) {
            if (LogSupport.isLoggable()) {
                LogSupport.log("MimetypesFileTypeMap: can't load " + name, sex);
            }
        }
        finally {
            try {
                if (clis != null) {
                    clis.close();
                }
            }
            catch (IOException ex) {}
        }
        return null;
    }
    
    private void loadAllResources(Vector v, String name) {
        boolean anyLoaded = false;
        try {
            ClassLoader cld = null;
            cld = SecuritySupport.getContextClassLoader();
            if (cld == null) {
                cld = this.getClass().getClassLoader();
            }
            URL[] urls;
            if (cld != null) {
                urls = SecuritySupport.getResources(cld, name);
            }
            else {
                urls = SecuritySupport.getSystemResources(name);
            }
            if (urls != null) {
                if (LogSupport.isLoggable()) {
                    LogSupport.log("MimetypesFileTypeMap: getResources");
                }
                int i = 0;
                while (i < urls.length) {
                    URL url = urls[i];
                    InputStream clis = null;
                    Block_10: {
                        if (LogSupport.isLoggable()) {
                            continue;
                        }
                        LogSupport.log("MimetypesFileTypeMap: URL " + url);
                        try {
                            clis = SecuritySupport.openStream(url);
                            if (clis != null) {
                                v.addElement(new MimeTypeFile(clis));
                                anyLoaded = true;
                                if (LogSupport.isLoggable()) {
                                    LogSupport.log("MimetypesFileTypeMap: successfully loaded mime types from URL: " + url);
                                }
                            }
                            else if (LogSupport.isLoggable()) {
                                LogSupport.log("MimetypesFileTypeMap: not loading mime types from URL: " + url);
                            }
                        }
                        catch (IOException ioex) {
                            if (LogSupport.isLoggable()) {
                                LogSupport.log("MimetypesFileTypeMap: can't load " + url, ioex);
                            }
                        }
                        catch (SecurityException sex) {
                            if (LogSupport.isLoggable()) {
                                LogSupport.log("MimetypesFileTypeMap: can't load " + url, sex);
                            }
                        }
                        finally {
                            try {
                                if (clis != null) {
                                    clis.close();
                                }
                            }
                            catch (IOException ex2) {}
                        }
                        ++i;
                    }
                }
            }
        }
        catch (Exception ex) {
            if (LogSupport.isLoggable()) {
                LogSupport.log("MimetypesFileTypeMap: can't load " + name, ex);
            }
        }
        if (!anyLoaded) {
            LogSupport.log("MimetypesFileTypeMap: !anyLoaded");
            MimeTypeFile mf = this.loadResource("/" + name);
            if (mf != null) {
                v.addElement(mf);
            }
        }
    }
    
    private MimeTypeFile loadFile(String name) {
        MimeTypeFile mtf = null;
        try {
            mtf = new MimeTypeFile(name);
        }
        catch (IOException ex) {}
        return mtf;
    }
    
    public MimetypesFileTypeMap(String mimeTypeFileName) throws IOException {
        this();
        this.DB[0] = new MimeTypeFile(mimeTypeFileName);
    }
    
    public MimetypesFileTypeMap(InputStream is) {
        this();
        try {
            this.DB[0] = new MimeTypeFile(is);
        }
        catch (IOException ex) {}
    }
    
    public synchronized void addMimeTypes(String mime_types) {
        if (this.DB[0] == null) {
            this.DB[0] = new MimeTypeFile();
        }
        this.DB[0].appendToRegistry(mime_types);
    }
    
    @Override
    public String getContentType(File f) {
        return this.getContentType(f.getName());
    }
    
    @Override
    public synchronized String getContentType(String filename) {
        int dot_pos = filename.lastIndexOf(".");
        if (dot_pos < 0) {
            return MimetypesFileTypeMap.defaultType;
        }
        String file_ext = filename.substring(dot_pos + 1);
        if (file_ext.length() == 0) {
            return MimetypesFileTypeMap.defaultType;
        }
        for (int i = 0; i < this.DB.length; ++i) {
            if (this.DB[i] != null) {
                String result = this.DB[i].getMIMETypeString(file_ext);
                if (result != null) {
                    return result;
                }
            }
        }
        return MimetypesFileTypeMap.defaultType;
    }
    
    static {
        MimetypesFileTypeMap.defaultType = "application/octet-stream";
    }
}
